#!/bin/bash
#

# buildXmlrpc-c.sh <product_dir> <e10|e14> <debug|opt|prof> [tar] [source]

# IMPORTANT: If curl-devel (SLF5) or libcurl-devel (SLF6) is installed, 
#            the curl xml client will also be built

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e14> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}/build
    tardir=${product_dir}/${package}/${pkgver}/tar
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=xmlrpc_c
origpkgver=v1_39_10
pkgver=${origpkgver}a
ssibuildshims_version=v0_15_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=xmlrpc-c-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e1[04] ]]; then
  cxxflg="${cflg} -std=c++14 -Wno-deprecated-declarations"
else
  die "Qualifier $basequal not recognized."
fi

if [ -d ${pkgdir} ]; then rm -rf ${pkgdir}; fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

# XMLRPC_C as used by ARTDAQ ALWAYS requires curl client...
#have_curl=`rpm -qa | grep curl-devel | wc -l` # Platform-dependent nastiness aside...
have_curl=1
if [ ${have_curl} -gt 0 ]
then
   build_curl=--enable-curl-client 
   echo "building xmlrpc_c with curl client"
else
   build_curl=--disable-curl-client 
   echo "building xmlrpc_c WITHOUT curl client"
fi

set -x

cd ${blddir}
tar xf ${tardir}/${pkgtarname}.tgz

cd ${blddir}/${pkgtarname}

./configure --prefix=${pkgdir} \
            ${build_curl} --enable-cplusplus \
	    --disable-libwww-client --disable-wininet-client || die "ERROR: configure failed."
##make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" all || die "ERROR: make failed."
## disable make check because the abyss.c test file explicitly uses port 8080 which is not always available
##make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" check || die "ERROR: make check failed."
make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" install || die "ERROR: make install failed."
# install the tools
cd ${blddir}/${pkgtarname}/tools; make CFLAGS_LOCAL="${cflg}" CXXFLAGS="${cxxflg}" install || die "ERROR: make install of tools failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "xmlrpc_c libraries are in ${XMLRPC_C_LIBRARY}"

# fixing embedded path
mv ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak
cat ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config.bak \
    | sed 's%^PREFIX=.*$%PREFIX=${XMLRPC_C_FQ_DIR}%' \
    | sed 's%^HEADERINST_DIR=.*$%HEADERINST_DIR=${XMLRPC_C_INCLUDE_DIR}%' \
    | sed 's%^LIBINST_DIR=.*$%LIBINST_DIR=${XMLRPC_C_LIBRARY}%' \
    | sed 's%^BLDDIR=.*$%BLDDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    | sed 's%^ABS_SRCDIR=.*$%ABS_SRCDIR=${XMLRPC_C_FQ_DIR}/build/stable%' \
    > ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config
chmod +x ${XMLRPC_C_FQ_DIR}/bin/xmlrpc-c-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
